"""
Small module to be used in the Use the Force! practicum at VU & UvA.
"""

from use_the_force._logging import *
from use_the_force.forceSensor import *
from use_the_force.plotting import *

__all__ = ["ForceSensor", "Logging", "Plotting", "Commands"]  # type: ignore



from time import perf_counter_ns, sleep
import serial

__all__ = ["ForceSensor", "Commands"]


class ForceSensor:
    def __init__(self, PortName: str | None = None, **kwargs) -> None:
        """
        Base class for the force sensor with almost all functionality.

        Use ForceSensor.cmds.`COMMAND_NAME`() for commands:
        >>> sensor = ForceSensor("COM0")
        >>> commands = sensor.cmds
        >>> commands.SR()
        411023

        :param PortName: Portname over which to establish the connection. If None, the this class has to be called again with the port name.
        :type PortName: str | None
        """
        # The 'zero' volt value. Determined automatically each time.
        self.tareValue: int = int(kwargs.pop("tareValue", 0))
        self.tareRound: int = int(kwargs.pop("tareRound", 0))
        self.loadPerCount: float = int(kwargs.pop("loadPerCount", 1.0))

        self.minPos: int = int(kwargs.pop("minPos", 1))  # [mm]
        self.maxPos: int = int(kwargs.pop("maxPos", 46))  # [mm]

        self.T0: int = perf_counter_ns()

        ####### PORT INIT ######
        # The 'COM'-port depends on which plug is used at the back of the computer.
        # To find the correct port: go to Windows Settings, Search for Device Manager,
        # and click the tab "Ports (COM&LPT)".s
        self.ser: serial.Serial = serial.Serial(
            port=PortName, baudrate=115200, timeout=5, dsrdtr=False
        )
        self.ser.setRTS(False)
        self.ser.setDTR(False)

        self.cmds = Commands(self.ser)

        if PortName is not None:
            self.PortName = PortName.upper()
            self.ser.setPort(self.PortName)
            self.ser.open()

    def __call__(self, PortName: str):
        """
        Open the serial connection if not already open.

        >>> sensor = ForceSensor()
        >>> sensor("COM0")

        :param PortName: Portname over which to establish the connection.
        :type PortName: str
        """
        if not self.ser.is_open:
            self.PortName = PortName.upper()
            self.ser.setPort(self.PortName)
            self.ser.open()

    def tare(self, reads: int = 30, skips: int = 3) -> int:
        """
        Updates and returns the tare value by taking the average of `reads` reads.

        :param reads: amount of readings
        :type reads: int
        :param skips: initial lines to skip (and clear old values)
        :type skips: int

        :returns: Tare value
        :rtype: int
        """
        self.ser.reset_input_buffer()
        self.ser.reset_output_buffer()
        skips: list[float] = [self.cmds.SR() for _ in range(skips)]
        read_values: list[float] = [self.cmds.SR() for _ in range(reads)]
        self.tareValue = round(sum(read_values) / reads, self.tareRound)
        return self.tareValue

    def updateLpC(self, load: float, reads: int = 10) -> float:
        """Updates and returns Load per Count

        :param load: Applied known load (Newton, gram, ...)
        :type load: float
        :param reads: Times to read load cell and take average
        :type reads: int

        :returns: Load per Count
        :rtype: float
        """
        read_values: list[float] = [self.cmd.SR() for i in range(reads)]
        self.loadPerCount = load / int(sum(read_values) / reads)
        return self.loadPerCount

    def ForceFix(self, count: float) -> float:
        """Corrects the units given based on tareValue and loadPerCount

        Args:
            count (float): sensor count

        Returns:
            float: calibrated units
        """
        # The output, with gauge, in calibrated units.
        return (count - self.tareValue) * self.loadPerCount

    def ClosePort(self) -> None:
        """
        Always close after use.
        """
        self.ser.close()


class Commands:
    def __init__(self, serialConnection: serial.Serial, **kwargs) -> None:
        """Class containing all available commands.

        For more explanation, see the firmware [GitHub](https://github.com/NatuurkundePracticumAmsterdam/use-the-force-firmware).

        :param serialConnection: serial connection to sensor
        :type serialConnection: Serial
        :param stdDelay: standard delay between sending a message and reading
        :type stdDelay: float
        """
        self.serialConnection: serial.Serial = serialConnection
        self.stdDelay: float = float(kwargs.pop("stdDelay", 0.0))

        self.cmdStart: str = "#"
        self.cmdArgSep: str = ","
        self.cmdEnd: str = ";"

        self.minPos: int = 1
        self.maxPos: int = 46

        self.verMajor: int = 0
        self.verMinor: int = 0
        self.verPatch: int = 0

    def __call__(self, serialConnection: serial.Serial) -> None:
        """Change serial connection

        :param serialConnection: new serial connection
        :type serialConnection: Serial
        """
        self.serialConnection = serialConnection

    def _clearBuffer(self) -> None:
        """
        Clears the serial buffer.
        """
        self.serialConnection.flush()
        self.serialConnection.reset_input_buffer()
        self.serialConnection.reset_output_buffer()

    def customCmd(self, cmd: str, *args) -> str:
        """Custom command

        Sends a self written command to the sensor.
        For general use, this command should not be used, as it is not guaranteed to work with the firmware.

        :param cmd: command to send
        :type cmd: str
        :param args: additional arguments for the command
        :type args: tuple

        :returns: return line
        :rtype: str
        """
        self.serialConnection.flush()
        cmdStr = f"{self.cmdStart}{cmd}"
        if len(args) != 0:
            cmdStr += f"{args[0]}"
            if len(args) - 1 != 0:
                for argument in args[1:]:
                    cmdStr += f"{self.cmdArgSep}{argument}"
        cmdStr += f"{self.cmdEnd}"
        self.serialConnection.write(cmdStr.encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        return returnLine

    ########################
    # 0 Arguments Commands #
    ########################
    def AB(self) -> None:
        """
        ### Abort CR

        Aborts the continous reading.

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}AB{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def CM(self) -> str:
        """
        ### Count Maximum

        Sets the currently read load as the maximum amount of force that is allowed.
        Will immediatly send out the abort message if load remains.

        This count is saved on the sensor.

        :raises RunTimeError: If sensor encounters an error.

        :returns: Returnline from sensor
        :rtype: str
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}CM{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        return returnLine

    def CZ(self) -> str:
        """
        ### Count Zero
        
        Tares the internal value count of the sensor, which is used to check the maximum force.\\
        Uses current load on the sensor as internal zero.
        
        This count is saved on the sensor. 
        
        :raises RunTimeError: If sensor encounters an error.

        :returns: Returnline from sensor
        :rtype: str
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}CZ{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        return returnLine

    def GP(self) -> int:
        """
        ### Get Position
        Current position set in memory.

        :return: End position if moving, else current position in [mm]
        :rtype: int

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}GP{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        else:
            try:
                return int(returnLine.split(": ")[-1])
            except ValueError as e:
                return e

    def GV(self) -> int:
        """
        ### Get Velocity
        Returns the current velocity of the steppermotor stage in milimeters per second.

        :return: End velocity if moving, else current velocity [mm/s]
        :rtype: int

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}GV{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        else:
            return int(returnLine.split(": ")[-1])

    def HE(self) -> ...:
        """
        ### Help

        Help command internally, not implemented here.

        :raises NotImplementedError: Not Implemented
        """
        raise NotImplementedError

    def HM(self) -> None:
        """
        ### Home
        Homes the steppermotor stage to the endstop.
        The endstop is a physical switch that stops the motor when it is pressed.
        Afterwards goes up to a set position inside the firmware.

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}HM{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def ID(self) -> str:
        """
        ### Motor ID

        Returns the ID that is set for the motor stage.

        :returns: Motor ID
        :rtype: str

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}ID{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        else:
            return returnLine

    def SR(self) -> float:
        """
        ### Single Read
        Reads the force a single time.

        :return: read force
        :rtype: float

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}SR{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        else:
            return float(returnLine.split(": ")[-1])

    def ST(self) -> None:
        """
        ### Force Stop

        Forces the motor to stop during movement.
        Will need to home afterwards.
        ### WARNING: DOES NOT WORK DURING HOME

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}ST{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def TR(self) -> None:
        """
        ### Tare

        Tares the display values by setting current reading as offset.
        Does not affect readings.

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}TR{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def VR(self) -> str:
        """
        ### Version

        Returns current running firmware version of the sensor.

        :returns: Firmware Version
        :rtype: str

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}VR{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        else:
            self.verMajor, self.verMinor, self.verPatch = map(
                int, returnLine.split(": ")[-1].split(".")
            )
            return returnLine

    #######################
    # 1 Argument Commands #
    #######################
    def DC(self, enable: bool = True) -> None:
        """
        ### Display Commands

        Enables or disables the display of commands on the sensor.

        :param enable: If commands should be displayed on sensor. Default: True
        :type enable: bool

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        if not enable:
            self.serialConnection.write(
                f"{self.cmdStart}DC false{self.cmdEnd}".encode()
            )
        else:
            self.serialConnection.write(f"{self.cmdStart}DC{self.cmdEnd}".encode())

        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def SF(self, calibrationForce: float) -> None:
        """
        ### Set Force

        Changes the display calibration, does not affect readings.
        Internally the force is stored as a float, so it is recommended to use a unit corresponding to the measurement range as to avoid floating point errors.

        :param calibrationForce: current force on the loadcell
        :type calibrationForce: float

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(
            f"{self.cmdStart}SF {calibrationForce}{self.cmdEnd}".encode()
        )
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def SP(self, position: int) -> None:
        """
        ### Set Position
        Sets the position of the steppermotor stage in milimeters.

        :param position: position to set from bottom [mm]
        :type position: int

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        if position <= self.maxPos and position >= self.minPos:
            self.serialConnection.flush()
            self.serialConnection.write(
                f"{self.cmdStart}SP{position}{self.cmdEnd}".encode()
            )
            if self.stdDelay > 0:
                sleep(self.stdDelay)
            returnLine: str = self.serialConnection.read_until().decode().strip()
            if returnLine.split(":")[0] == "[ERROR]":
                raise RuntimeError(returnLine)
        else:
            raise ValueError(
                f"Position {position} is out of range ({self.minPos}, {self.maxPos})"
            )

    def SV(self, velocity: int) -> None:
        """
        ### Set Velocity
        Sets the velocity of the steppermotor stage in milimeters per second.

        :param velocity: velocity to set [mm/s]
        :type velocity: int

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(
            f"{self.cmdStart}SV{velocity}{self.cmdEnd}".encode()
        )
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def UL(self, lineHeight: int) -> None:
        """
        ### Update Text Line Height

        Changes the line height set in the sensor.

        :param lineHeight: current force on the loadcell
        :type lineHeight: int

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(
            f"{self.cmdStart}UL{lineHeight}{self.cmdEnd}".encode()
        )
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def UU(self, unit: str) -> None:
        """
        ### Update Unit Displayed

        Changes the unit displayed on the interface.

        :param unit: new unit, max 8 chars.
        :type unit: str

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}UU{unit}{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def UX(self, xOffset: int) -> None:
        """
        ### Update display x offset

        Updates the x offset of the display.

        :param xOffset: new offset
        :type xOffset: int

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}UX{xOffset}{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    def UY(self, yOffset: int) -> None:
        """
        ### Update display y offset

        Updates the y offset of the display.

        :param yOffset: new offset
        :type yOffset: int

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(f"{self.cmdStart}UY{yOffset}{self.cmdEnd}".encode())
        if self.stdDelay > 0:
            sleep(self.stdDelay)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)

    ########################
    # 2 Arguments Commands #
    ########################
    def CR(self, nReads: int, iReads: int) -> list[list]:
        """
        ### Continuous Reading
        Reads nReads times the force with an iReads interval inbetween.

        :param nReads: number of lines to read
        :type nReads: int
        :param iReads: interval inbetween lines [ms]
        :type iReads: int

        :return: [[time], [force]]
        :rtype: list[list[int], list[float]]

        :raises RunTimeError: If sensor encounters an error.
        """
        self._clearBuffer()
        self.serialConnection.write(
            f"{self.cmdStart}CR {nReads}{self.cmdArgSep}{iReads}{self.cmdEnd}".encode()
        )
        sleep(self.stdDelay + iReads / 1000)
        returnLine: str = self.serialConnection.read_until().decode().strip()
        if returnLine.split(":")[0] == "[ERROR]":
            raise RuntimeError(returnLine)
        else:
            time, force = returnLine.split(": ")[-1].split(";")
            time = int(time)
            force = float(force)
            currentReads = [[time], [force]]

            for i in range(nReads):
                returnLine = self.serialConnection.read_until().decode().strip()
                if returnLine.split(":")[0] == "[ERROR]":
                    raise RuntimeError(returnLine)
                else:
                    time, force = returnLine.split(": ")[-1].split(",")
                    time = int(time)
                    force = float(force)
                    currentReads[0].append(time)
                    currentReads[1].append(force)
            return currentReads



from io import TextIOWrapper

__all__ = ["Logging"]


class Logging:
    def __init__(
        self, filename: str = "", NeverCloseFile: bool = False, extension: str = ".csv"
    ) -> None:
        """
        Class to log the data from the force sensor

        Allows for multiple measurements to be taken with the files increasing the `_i` identifier.
        """
        self.filename: str = filename
        self.full_filename: str
        self.HAND: TextIOWrapper
        self.NeverCloseFile: bool = NeverCloseFile
        self.extension: str = extension

    def createLog(self, ext: str = ".csv") -> None:
        """
        Creates a new file for logging.
        """

        # Check for a file that does not exist yet.
        i = 0
        self.full_filename = "DATA/" + self.filename + "_" + str(i) + ext
        while True:
            try:
                f = open(self.full_filename, "r")
                f.close()
                i += 1
                self.full_filename = "DATA/" + self.filename + "_" + str(i) + ext
            except:
                break

        # Create this file.
        self.HAND = open(self.full_filename, "w+")

        self.HAND.write("Time,Displacement,Force\n")

        if not self.NeverCloseFile:
            self.HAND.close()

    def createLogGUI(self) -> None:
        """
        Creates a new file for logging, GUI variant.
        """

        # Check for a file that does not exist yet.
        self.full_filename = self.filename

        # Create this file.
        self.HAND = open(self.full_filename, "w+")
        self.HAND.write("Time,Displacement,Force\n")
        self.HAND.close()
        if self.NeverCloseFile:
            self.HAND = open(self.full_filename, "a+")

    def replaceFile(self, data: list[float | int]):
        self.HAND = open(self.full_filename, "w+t")
        self.NeverCloseFile = True
        self.writeLogFull(data=data)
        self.NeverCloseFile = False
        self.HAND.close()

    ### ===LOGGING FUNCTION===###
    # Puts the values in the given list into the opened log file.
    def writeLog(self, data: list[float | int]) -> None:
        # Open file
        if not self.NeverCloseFile:
            self.HAND = open(self.full_filename, "a+")

        # Write data
        for i, d in enumerate(data):
            if i == 0:
                txt = str(d)
            else:
                txt = str(round(d, 8))
                self.HAND.write(",")

            self.HAND.write(txt)
        self.HAND.write("\n")

        # Close file
        if not self.NeverCloseFile:
            self.HAND.close()

    def writeLogFull(self, data: list[float | int]) -> None:
        # Open file
        if not self.NeverCloseFile:
            self.HAND = open(self.full_filename, "a+")
        # Write data, variable length of `data`
        for indexData in range(len(data[0])):
            line: str = str()
            lineValues: list[int | float] = []

            for indexUnit in range(len(data)):
                lineValues.append(str(data[indexUnit][indexData]))
            line = ",".join(lineValues) + "\n"
            self.HAND.write(line)

        # Close file
        if not self.NeverCloseFile:
            self.HAND.close()

    ### ===READ LOG===###
    def readLog(self, *, filename: str | None = None) -> list[list[float]]:
        if filename is None:
            filename = self.filename

        if self.NeverCloseFile and filename is not None:
            file = self.HAND
        else:
            file = open(filename, "r")

        data = [[], [], []]
        for line in file[1:]:
            t, s, F = line.strip().split(",")
            data[0].append(float(t))
            data[1].append(float(s))
            data[2].append(float(F))

        file.close()
        return data

    ### ===MANUAL CLOSING FUNCTION===###
    # Closes file, irregardless of whether 'NeverCloseFile' is True.
    def closeFile(self) -> None:
        if self.NeverCloseFile:
            self.HAND.close()
        else:
            pass  # file should be closed already



"""
GUI module to launch the GUI.
"""

from use_the_force.gui.gui import *
from use_the_force.gui.error_ui import *
from use_the_force.gui.main_ui import *

__all__ = [
    "UserInterface",
    "mainLogWorker",
    "saveToLog",
    "ForceSensorGUI",
    "ErrorInterface",
    "start",
    "Ui_MainWindow",
    "Ui_errorWindow",
]  # type: ignore



from matplotlib import pyplot as plt
import matplotlib

# Use TkAgg backend for interactive plotting
# TkAgg is way less laggy than the default Agg backend
matplotlib.use("TkAgg")
# self.plt.ion()

__all__ = ["Plotting"]


class Plotting:
    def __init__(
        self, xlabel: str = "Time (s)", ylabel: str = "Force (mN)", **kwargs
    ) -> None:
        """Class to start with plots.

        :param xlabel: Text displayed on x-axes
        :type xlabel: str
        :param ylabel: Text displayed on y-axes
        :type ylabel: str
        :param startTime: Offsets xlimit of the plot
        :type startTime: int
        :rtype: None
        """
        ### ===PARAMETERS THAT ONE CAN ALTER=== ###
        self.xlabel: str = xlabel
        self.ylabel: str = ylabel
        self.startTime: float = float(kwargs.pop("startTime", 0.0))

        ### ===START A NEW FIG=== ###
        self._init_fig()

    def _init_fig(self) -> None:
        """
        Initializes a new figure
        """
        # 1: Create plot
        self.fig, self.ax1 = plt.subplots(1)
        (self.lines,) = self.ax1.plot([], [])

        # 2: Making the axis prettier.
        # self.ax1.set_autoscalex_on(True)
        self.ax1.set_xlabel(self.xlabel)
        self.ax1.set_ylabel(self.ylabel)
        # self.ax1.set_autoscaley_on(True)
        # self.ax1.set_ylim( self.MinY, self.MaxY )
        self.ax1.grid(visible=True)
        self.ax1.set_xlim(left=self.startTime)

        # 3: Alternative text boxes, will report the time between two points,
        # and all the current heights.
        # self.txtR = self.ax1.text( 0, self.MinY , "" )

        self.fig.show()
        self.fig.canvas.draw()

    ### ===ANIMATION FUNCTION===###
    # Each loop this function is performed. I return the graph.

    def Update(self, data) -> None:
        """
        Updates the canvas to contain new data

        Replaces the entire data set with the new one, should be fine for smaller data sets.

        :param data: The new data set to be drawn
        :type data: list
        :rtype: None
        """
        # Set new data
        self.lines.set_data(data)

        self.ax1.set_ylim(
            bottom=min(data[1]) - abs(min(data[1])) / 10,
            top=max(data[1]) + abs(max(data[1])) / 10,
        )
        self.ax1.set_xlim(left=self.startTime, right=data[0][-1])
        self.ax1.autoscale_view()
        # We need to draw *and* flush
        self.fig.canvas.draw()
        self.fig.canvas.flush_events()



import bisect
import re
import sys
import threading
from time import perf_counter_ns, sleep

import pyqtgraph as pg
from PySide6 import QtWidgets
from PySide6.QtCore import QObject, QRunnable, Qt, QThreadPool, QTimer, Signal, Slot
from PySide6.QtGui import QCloseEvent
from serial.tools import list_ports  # type: ignore

from use_the_force._logging import Logging
from use_the_force.forceSensor import ForceSensor
from use_the_force.gui.error_ui import Ui_errorWindow
from use_the_force.gui.main_ui import Ui_MainWindow

__all__ = [
    "UserInterface",
    "mainLogWorker",
    "saveToLog",
    "ForceSensorGUI",
    "ErrorInterface",
    "start",
]


class UserInterface(QtWidgets.QMainWindow):
    def __init__(self) -> None:
        super().__init__()

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        # disable MDM until switched
        self.ui.MDM.setVisible(False)
        self.ui.MDM.setEnabled(False)

        self.error_ui = ErrorInterface()

        ###########
        # BUTTONS #
        ###########
        # Button event mapping
        self.button_handlers = {
            self.ui.butConnect: self.butConnect,
            self.ui.butFile: self.butFile,
            self.ui.butTare: self.butTare,
            self.ui.butRecord: self.butRecord,
            self.ui.butClear: self.butClear,
            self.ui.butSave: self.butSave,
            self.ui.butSingleRead: self.butSingleRead,
            self.ui.butSwitchManual: self.butSwitchMDM,
            self.ui.butFileMDM: self.butFileMDM,
            self.ui.butReadForceMDM: self.readForceMDM,
            self.ui.butSwitchDirectionMDM: self.switchDirectionMDM,
            self.ui.butDeletePreviousMDM: self.butDeletePreviousMDM,
            self.ui.butMove: self.butMove,
            self.ui.butUpdateVelocity: self.butUpdateVelocity,
            self.ui.butHome: self.butHome,
            self.ui.butForceStop: self.butForceStop,
            self.ui.butTareDisplay: self.butDisplayTare,
            self.ui.butSwapPositions: self.swapPositions,
        }

        # Connect all buttons
        for button, handler in self.button_handlers.items():
            button.pressed.connect(handler)

        # Text boxes and value boxes
        self.ui.setNewtonPerCount.valueChanged.connect(self.setLoadPerCount)
        self.ui.setPlotTimerInterval.valueChanged.connect(self.updatePlotTimerInterval)
        self.ui.setLineReads.valueChanged.connect(self.singleReadLinesForcesUpdate)
        self.ui.setLineSkips.valueChanged.connect(self.singleReadSkipsUpdate)
        self.ui.setStepSizeMDM.valueChanged.connect(self.singleReadStepUpdate)
        self.ui.title_2.textChanged.connect(self.updatePlotMDMTitle)
        self.ui.setUnitDisplay.editingFinished.connect(self.updateUnitDisplay)

        ###############
        # Check Ports #
        ###############
        ports: list[str] = [port.device for port in list_ports.comports()]
        if len(ports) > 0:
            self.ui.setPortName.setText(ports[0])
        else:
            self.ui.setPortName.setText("No ports found")
        del ports

        ###################
        # INITIALIZE VARS #
        ###################
        self.sensorConnected: bool = False
        self.threadReachedEnd: bool = False
        self.recording: bool = False
        self.fileGraphOpen: bool = False
        self.fileOpen: bool = False
        self.fileMDMOpen: bool = False
        self.readForceMDMToggle: bool = False
        self.switchDirectionMDMToggle: bool = False
        self.MDMActive: bool = False
        self.singleReadToggle: bool = False
        self.homed: bool = False
        self.singleReadForce: float = float()
        self.singleReadForces: int = self.ui.setLineReads.value()
        self.singleReadSkips: int = self.ui.setLineSkips.value()
        self.stepSizeMDM: float = self.ui.setStepSizeMDM.value()
        self.plotIndexX: int = 1
        self.plotIndexY: int = 2
        self.velocity: int = self.ui.setVelocity.value()
        self.txtLogMDM: str = str()
        self.reMDMMatch: re.Pattern[str] = re.compile(r"\[[A-Za-z0-9]+\]")
        self.data: list[list[float]] = [[], [], []]
        setattr(self.ui, "errorMessage", [])

        ###################
        # INITIALIZE PLOT #
        ###################
        self.plot(clrBg="default")
        self.plotMDM()

        # Plot timer interval in ms
        self.plotTimerInterval: int = self.ui.setPlotTimerInterval.value()

        ##################
        # MULTITHREADING #
        ##################
        self.sensor = ForceSensorGUI(caller=self)
        # self.cmds = Commands(self.sensor.ser)
        self.sensor.errorSignal.connect(self.error)

        self.plotTimer = QTimer()
        self.plotTimer.timeout.connect(self.updatePlot)

        self.mainLogWorker = mainLogWorker(self)
        self.mainLogWorker.startSignal.connect(self.startPlotTimer)
        self.mainLogWorker.endSignal.connect(self.stopPlotTimer)
        self.mainLogWorker.switchXAxisSignal.connect(self.switchToTime)
        # self.mainLogWorker.singleReadStartSignal.connect()
        self.mainLogWorker.singleReadEndSignal.connect(self.singleReadEnd)

        self.saveToLog = saveToLog(self)
        self.saveToLog.startSignal.connect(self.saveStart)
        self.saveToLog.endSignal.connect(self.saveEnd)

        self.thread_pool = QThreadPool.globalInstance()

        ############################
        # CHANGE IN NEXT UI UPDATE #
        ############################
        # TODO: add screen for movement options and movement cycles.
        # ^ Might never update this one ^

    def enableElement(self, *elements: QtWidgets.QWidget) -> None:
        """
        Enables the specified GUI elements.
        """
        [element.setEnabled(True) for element in elements]

    def disableElement(self, *elements: QtWidgets.QWidget) -> None:
        """
        Disables the specified GUI elements.
        """
        [element.setEnabled(False) for element in elements]

    def resetConnectUI(self) -> None:
        """
        Resets the UI state for the connect button and related elements.
        """
        self.ui.butConnect.setText("Connect")
        self.ui.butConnect.setChecked(False)
        self.sensorConnected = False
        self.homed = False
        self.enableElement(self.ui.butConnect, self.ui.setPortName)
        self.ui.setForceApplied.editingFinished.disconnect(self.butDisplayForce)
        self.disableElement(
            self.ui.setNewtonPerCount,
            self.ui.setGaugeValue,
            self.ui.butRecord,
            self.ui.butTare,
            self.ui.butSingleRead,
            self.ui.setUnitDisplay,
            self.ui.butTareDisplay,
            self.ui.setForceApplied,
            self.ui.butHome,
        )
        self.ui.toolBox.setItemText(
            self.ui.toolBox.indexOf(self.ui.sensorOptions), "Sensor"
        )

    def setConnectedUI(self) -> None:
        """
        Updates the UI state for a successful connection.
        """
        self.ui.butConnect.setText("Connected")
        self.ui.butConnect.setChecked(True)
        self.enableElement(
            self.ui.butTare,
            self.ui.butSingleRead,
            self.ui.setNewtonPerCount,
            self.ui.setGaugeValue,
            self.ui.butHome,
            self.ui.butForceStop,
            self.ui.butUpdateVelocity,
            self.ui.butConnect,
            self.ui.setUnitDisplay,
            self.ui.butTareDisplay,
            self.ui.setForceApplied,
        )
        self.disableElement(self.ui.setPortName)
        self.ui.setForceApplied.editingFinished.connect(self.butDisplayForce)
        if not self.MDMActive:
            if not self.fileOpen:
                self.butClear()
            self.enableElement(self.ui.butFile)
        elif self.fileMDMOpen:
            self.enableElement(self.ui.butReadForceMDM)

    def closeEvent(self, event: QCloseEvent) -> None:
        """
        Safely closes the program and ends certain threads

        Some threads can be set to run infinitly, but will now be closed
        """
        if self.recording:
            self.recording = False
        if self.sensorConnected:
            self.butConnect()
        if not (self.fileOpen or self.MDMActive) and len(self.data[0]) > 0:
            self.ui.errorMessage = [
                "Unsaved Data",
                "Unsaved Data!",
                "You have unsaved data, quit anyways?",
            ]
            if not self.error():  # Cancel
                event.ignore()
                self.butSave()

    def plot(self, **kwargs) -> None:
        """
        Plots the data on the central plot.

        :param data: list containing both x-axes and y-axes as `[...,x,y]`. `...` is ignored
        :type data: list[..., list, list]

        :param label1loc: location of first label, default: `"left"`
        :type label1loc: str
        :param label1txt: text of first label
        :type label1txt: str
        :param label2loc: location of second label, default: `"bottom"`
        :type label2loc: str
        :param label2txt: text of second label
        :type label2txt: str

        :param color: line color, default: `"r"`
        :type color: str
        :param linewidth: linewidth, default: `5`
        :type linewidth: int

        :param clrBg: color of the background, default: `"w"`
        :type clrBg: str
        :param clrFg: color of the foreground, default: `"k"`
        :type clrFg: str
        """

        pg.setConfigOption("foreground", kwargs.pop("clrFg", "k"))
        pg.setConfigOption("background", kwargs.pop("clrBg", "w"))
        # self.ui.graphMDM.setBackground(background=kwargs.pop("clrBg", "w"))
        self.ui.graph1.plot(
            *self.data[-2:],
            symbol=kwargs.pop("symbol", None),
            pen={
                "color": kwargs.pop("color", "r"),
                "width": kwargs.pop("linewidth", 5),
            },
        )

        self.updatePlotLabel(
            graph=self.ui.graph1,
            labelLoc=kwargs.pop("label1loc", "left"),
            labelTxt=kwargs.pop("label1txt", self.ui.yLabel.text()),
        )

        self.updatePlotLabel(
            graph=self.ui.graph1,
            labelLoc=kwargs.pop("label2loc", "bottom"),
            labelTxt=kwargs.pop("label2txt", "Displacement [mm]"),
        )

        self.ui.yLabel.textChanged.connect(self.updatePlotYLabel)

        self.ui.graph1.setTitle(self.ui.title.text(), color=(255, 255, 255))
        self.ui.title.textChanged.connect(self.updatePlotTitle)

    def updatePlot(self) -> None:
        """
        Updates the plot
        """
        self.ui.graph1.plot(self.data[self.plotIndexX], self.data[self.plotIndexY])

        if len(self.data[self.plotIndexX]) > 0:
            try:
                self.xLim = int(self.ui.xLimSet.value())
                if abs(self.xLim) < self.data[self.plotIndexX][-1] and (self.xLim != 0):
                    self.ui.graph1.setXRange(
                        self.data[self.plotIndexX][-1] + self.xLim,
                        self.data[self.plotIndexX][-1],
                    )
                    i = bisect.bisect_left(
                        self.data[self.plotIndexX],
                        self.data[self.plotIndexX][-1] + self.xLim,
                    )
                    self.ui.graph1.setYRange(
                        min(self.data[self.plotIndexY][i:]),
                        max(self.data[self.plotIndexY][i:]),
                    )

                elif self.xLim == 0:
                    self.ui.graph1.setXRange(0, self.data[self.plotIndexX][-1])
                    self.ui.graph1.setYRange(
                        min(self.data[self.plotIndexY]), max(self.data[self.plotIndexY])
                    )

            except:
                self.ui.graph1.setXRange(0, self.data[self.plotIndexX][-1])
                self.ui.graph1.setYRange(
                    min(self.data[self.plotIndexY]), max(self.data[self.plotIndexY])
                )

    def switchPlotIndexX(self, index: int) -> None:
        self.plotIndexX = index
        if index == 0:
            self.updatePlotLabel(
                graph=self.ui.graph1, labelLoc="bottom", labelTxt="Time [s]"
            )
        elif index == 1:
            self.updatePlotLabel(
                graph=self.ui.graph1, labelLoc="bottom", labelTxt="Displacement [mm]"
            )
        self.ui.graph1.clear()
        self.updatePlot()

    def switchPlotIndexY(self, index: int) -> None:
        self.plotIndexY = index
        self.ui.graph1.clear()
        self.updatePlot()

    def switchToTime(self) -> None:
        self.switchPlotIndexX(0)

    def updatePlotLabel(self, graph, labelLoc: str, labelTxt: str) -> None:
        """
        Updates the label

        :param graph: what graph to update
        :type PlotWidget:
        :param labelLoc: label location
        :type labelLoc: str
        :param labelTxt: label text
        :type labelTxt: str
        """
        graph.setLabel(labelLoc, labelTxt)

    def updatePlotYLabel(self) -> None:
        self.updatePlotLabel(
            graph=self.ui.graph1, labelLoc="left", labelTxt=self.ui.yLabel.text()
        )

    def updatePlotTimerInterval(self) -> None:
        tmp = self.ui.setPlotTimerInterval.value()
        try:
            tmp = int(tmp)
            if tmp > 0:
                self.plotTimerInterval = tmp
                if hasattr(self, "plotTimer"):
                    self.plotTimer.setInterval(self.plotTimerInterval)

        except:
            pass

        del tmp

    def updatePlotTitle(self) -> None:
        self.ui.graph1.setTitle(self.ui.title.text(), color=(255, 255, 255))

    def startPlotTimer(self) -> None:
        """
        Start the QTimer in the main thread when the signal is emitted.
        """
        self.plotTimer.start()

    def stopPlotTimer(self) -> None:
        """
        Stop the QTimer
        """
        self.plotTimer.stop()

    def butConnect(self) -> None:
        """
        Function defining what to do when a button is pressed.

        - checks if butConnect isChecked()
        - Starts a thread to connect/ disconnect the sensor
        - Thread ends with re-enabling the button
        """
        # Gets enabled again at the end of the thread
        self.ui.butConnect.setEnabled(False)

        # Disconnect
        if self.sensorConnected:
            self.sensorConnected = False

            self.startsensorDisonnect = threading.Thread(
                target=self.sensorDisconnect, name="sensorDisconnect"
            )
            self.startsensorDisonnect.start()
            self.ui.setPortName.setEnabled(True)

        # Connect
        else:
            devices: list[str] = [port.device for port in list_ports.comports()]
            if self.ui.setPortName.text() in devices:
                self.sensorConnected = True
                self.ui.butFile.setEnabled(False)
                self.startsensorConnect = threading.Thread(
                    target=self.sensorConnect, name="sensorConnect"
                )
                self.startsensorConnect.start()
            else:
                if len(devices) > 0:
                    self.ui.errorMessage = [
                        "Port not found",
                        f"Port: {self.ui.setPortName.text().upper()} was not detected!",
                        "Available ports:\n"
                        + "\n".join([port.device for port in list_ports.comports()]),
                    ]
                    self.error()
                else:
                    self.ui.errorMessage = [
                        "Port not found",
                        f"Port: {self.ui.setPortName.text().upper()} was not detected!",
                        "Available ports:\nNo ports found!",
                    ]
                    self.error()
                self.ui.butConnect.setText("Connect")
                self.ui.butConnect.setEnabled(True)
            del devices

    def sensorConnect(self) -> None:
        """
        Script to connect to the M5Din Meter.

        If connection fails, will raise an error dialog with the error.
        """
        self.ui.butConnect.setText("Connecting...")
        self.sensor()
        if self.sensor.failed:
            self.sensor.failed = False
            self.resetConnectUI()
            return

        # needs time or it will break
        sleep(0.5)
        try:
            vr = self.sensor.cmds.VR()
            if vr == "":
                raise RuntimeError("[ERROR]: Returned empty string.")
            else:
                self.ui.toolBox.setItemText(
                    self.ui.toolBox.indexOf(self.ui.sensorOptions),
                    "Sensor v:" + vr.split(":")[1][1:],
                )
        except RuntimeError:
            self.sensor.ClosePort()
            self.resetConnectUI()
            self.ui.errorMessage = [
                "Connection Error",
                "Connection Error",
                "[ERROR]: Retrieved no data.",
            ]
            self.error()
            return

        pos = self.sensor.cmds.GP()
        vel = self.sensor.cmds.GV()
        self.ui.setVelocity.setValue(vel)
        self.velocity = vel
        if pos >= 0 and pos < 47:
            self.homed = True
            self.enableElement(self.ui.butRecord, self.ui.butMove)
            self.ui.setPosition.setValue(pos)

        self.setConnectedUI()

    def sensorDisconnect(self) -> None:
        """
        Script to safely disconnect the M5Din Meter.

        Will first stop the recording, if running, with `butRecord()` function.
        """
        if self.recording:
            self.butRecord()
        self.sensor.ClosePort()
        sleep(0.5)  # Give some time to Windows/M5Din Meter to fully disconnect
        self.resetConnectUI()

    @Slot(str, str, str)
    @Slot(str, str, None)
    def error(self) -> bool:
        """
        Launches the error dialog.

        :param errorType: error type name, can be found with `Exception.__class__.__name__`
        :type errorType: str
        :param errorText: text why the error occured
        :type errorText: str

        :returns: Result of dialogue (button pressed), `True` for OK, `False` for Cancel
        :rtype: bool
        """
        if len(self.ui.errorMessage) == 3:
            if self.ui.errorMessage[2] == "[ERROR]: movement aborted, home to unlock":
                if self.recording:
                    self.recording = False
                    self.ui.butRecord.setText("Start")
                    self.enableElement(
                        self.ui.butClear,
                        self.ui.butFile,
                        self.ui.butSave,
                        self.ui.butSwitchManual,
                    )
                self.homed = False
                self.enableElement(self.ui.butHome)
                self.disableElement(self.ui.butRecord, self.ui.butMove)

        return bool(self.error_ui(*self.ui.errorMessage))

    def butFile(self) -> None:
        """
        Function for what `butFile` has to do.

        What to do is based on if the button is in the `isChecked()` state.
        - `if isChecked():` close file
        - `else:` opens dialog box to select/ create a .csv file
        """
        if self.fileOpen:
            self.fileOpen = False
            self.ui.butFile.setChecked(True)
            self.measurementLog.closeFile()
            del self.measurementLog
            self.ui.butFile.setText("-")
            self.butClear()
            self.ui.butSwitchManual.setEnabled(True)

        else:
            self.fileOpen = True
            self.ui.butFile.setChecked(True)
            self.filePath, _ = QtWidgets.QFileDialog.getSaveFileName(
                filter="CSV files (*.csv)"
            )
            # Cancel gives a 0 length string
            if self.filePath != "":
                self.measurementLog = Logging(self.filePath)
                self.measurementLog.createLogGUI()
                self.ui.butFile.setText(*self.filePath.split("/")[-1].split(".")[:-1])
                if len(self.data[1]) > 0:
                    self.ui.butSave.setEnabled(False)
                    self.thread_pool.start(self.saveToLog.run)
                self.ui.butSwitchManual.setEnabled(False)
            else:
                self.fileOpen = False
                self.ui.butFile.setText("-")
                self.ui.butFile.setChecked(False)

    def butRecord(self) -> None:
        """
        start button, disables/ enables most buttons and starts/ joins threads for the logging
        """
        if self.recording:
            self.recording = False
            self.ui.butRecord.setText("Start")
            self.enableElement(
                self.ui.butClear,
                self.ui.butFile,
                self.ui.butTare,
                self.ui.butSave,
                self.ui.butSingleRead,
                self.ui.butSwitchManual,
                self.ui.butUpdateVelocity,
                self.ui.butHome,
            )

            if not self.threadReachedEnd:
                pass  # one day will be able to wait for motor to stop.

        else:
            if len(self.data[0]) > 0:
                if not self.fileOpen:
                    self.ui.errorMessage = [
                        "Unsaved Data",
                        "Unsaved Data!",
                        "You have unsaved data, do you wish to continue?",
                    ]
                    if not self.error():  # Cancel
                        self.butSave()
                        return
                self.butClear()

            self.recording = True
            self.threadReachedEnd = False
            self.ui.butRecord.setText("Stop")
            self.ui.butRecord.setChecked(True)

            self.disableElement(
                self.ui.butClear,
                self.ui.butFile,
                self.ui.butTare,
                self.ui.butSave,
                self.ui.butSingleRead,
                self.ui.butSwitchManual,
                self.ui.butUpdateVelocity,
                self.ui.butHome,
            )

            self.sensor.ser.reset_input_buffer()

            if (
                self.ui.setStartPos.value() == self.ui.setEndPos.value()
                and self.plotIndexX != 0
                and self.ui.setTime.value() > 0
            ):
                self.switchPlotIndexX(0)
            elif self.plotIndexX != 1:
                self.switchPlotIndexX(1)

            self.mainLogWorker.logLess = self.ui.butFile.text() == "-"
            self.thread_pool.start(self.mainLogWorker.run)

    def butClear(self) -> None:
        """
        button that clears data in `self.data` and resets graph
        """
        del self.data
        self.data = [[], [], []]
        if self.MDMActive:
            self.graphMDM1.clear()
            self.graphMDM2.clear()
        else:
            self.ui.graph1.clear()
        if self.sensorConnected:
            self.sensor.ser.reset_input_buffer()
        self.ui.butSave.setEnabled(False)
        if self.fileOpen:
            self.butFile()

    def butTare(self) -> None:
        """
        button for Taring values sent from the M5Din Meter

        starts a thread to count down, end of thread re-enables button
        """
        self.disableElement(
            self.ui.butTare,
            self.ui.butConnect,
            self.ui.butRecord,
            self.ui.butSingleRead,
        )
        th = threading.Thread(target=self.butTareActive, name="tareActive")
        th.start()

    def butTareActive(self) -> None:
        """
        the actual Tare script
        """
        self.ui.butTare.setChecked(True)

        self.ui.butTare.setText("...")
        GaugeValue = self.sensor.tare()
        self.ui.setGaugeValue.setValue(GaugeValue)
        self.sensor.tareValue = GaugeValue
        self.ui.butTare.setText("Tare")

        if (not self.MDMActive) and self.homed:
            self.enableElement(self.ui.butRecord)
        self.enableElement(self.ui.butTare, self.ui.butConnect, self.ui.butSingleRead)
        self.ui.butTare.setChecked(False)

    def butSave(self) -> None:
        """
        Function for what `butSave` has to do.

        What to do is based on if `butFile` is in the `isChecked()` state.
        - `if isChecked():` do nothing as it is already saved
        - `else:` open new file and write data
        """
        if self.fileOpen:
            # When a file is selected it will already
            # write to the file when it reads a line
            self.disableElement(self.ui.butSave)
            self.ui.toolBox.setCurrentIndex(2)

        else:
            self.butFile()
            # Cancelling file selecting gives a 0 length string
            if self.filePath != "":
                self.disableElement(self.ui.butSave)
                self.ui.toolBox.setCurrentIndex(2)

    def saveStart(self) -> None:
        self.ui.butSave.setText("Saving...")

    def saveEnd(self) -> None:
        self.ui.butSave.setText("Save")

    def butSingleRead(self) -> None:
        self.singleReadToggle = True
        self.disableElement(
            self.ui.butSingleRead,
            self.ui.butRecord,
            self.ui.butConnect,
            self.ui.butTare,
        )
        self.thread_pool.start(self.mainLogWorker.singleRead)

    def singleReadEnd(self) -> None:
        if self.MDMActive:
            if self.fileMDMOpen:
                self.enableElement(self.ui.butReadForceMDM)
            if self.singleReadToggle:
                self.ui.SRCount.setText("{:.5f}".format(self.singleReadForce))
                self.singleReadToggle = False
            else:
                if self.readForceMDMToggle:
                    self.data[0].append(0)
                    self.data[1].append(
                        round(
                            self.data[1][-1] + self.stepSizeMDM,
                            len(str(self.stepSizeMDM).split(".")[-1]),
                        )
                    )
                    self.data[2].append(self.singleReadForce)

                    if re.search(
                        self.reMDMMatch, self.ui.xLabel_2.text()
                    ) and re.search(self.reMDMMatch, self.ui.yLabel_2.text()):
                        xUnit: list[str] = (
                            self.ui.xLabel_2.text().split("[")[-1].split("]")
                        )
                        yUnit: list[str] = (
                            self.ui.yLabel_2.text().split("[")[-1].split("]")
                        )
                        if len(xUnit) > 0 and len(yUnit) > 0:
                            self.txtLogMDM = (
                                self.txtLogMDM
                                + f"\n{self.data[1][-1]} {xUnit[0]}, {self.data[2][-1]} {yUnit[0]}"
                            )
                        else:
                            self.txtLogMDM = (
                                self.txtLogMDM
                                + f"\n{self.data[1][-1]}, {self.data[2][-1]}"
                            )
                    else:
                        self.txtLogMDM = (
                            self.txtLogMDM + f"\n{self.data[1][-1]}, {self.data[2][-1]}"
                        )
                    self.ui.plainTextEdit.setPlainText(self.txtLogMDM)
                    self.plainTextEditScrollbar = (
                        self.ui.plainTextEdit.verticalScrollBar()
                    )
                    self.plainTextEditScrollbar.setValue(
                        self.plainTextEditScrollbar.maximum()
                    )
                else:
                    self.data[0].append(0)
                    self.data[1].append(0.0)
                    self.data[2].append(self.singleReadForce)
                    self.readForceMDMToggle = True
                    if re.search(
                        self.reMDMMatch, self.ui.xLabel_2.text()
                    ) and re.search(self.reMDMMatch, self.ui.yLabel_2.text()):
                        xUnit: list[str] = (
                            self.ui.xLabel_2.text().split("[")[1].split("]")
                        )
                        yUnit: list[str] = (
                            self.ui.yLabel_2.text().split("[")[1].split("]")
                        )
                        if len(xUnit) > 0 and len(yUnit) > 0:
                            self.txtLogMDM = (
                                self.txtLogMDM
                                + f"{self.data[1][-1]} {xUnit[0]}, {self.data[2][-1]} {yUnit[0]}"
                            )
                    else:
                        self.txtLogMDM = (
                            self.txtLogMDM + f"{self.data[1][-1]}, {self.data[2][-1]}"
                        )
                    self.ui.plainTextEdit.setPlainText(self.txtLogMDM)
                    self.plainTextEditScrollbar = (
                        self.ui.plainTextEdit.verticalScrollBar()
                    )
                    self.plainTextEditScrollbar.setValue(
                        self.plainTextEditScrollbar.maximum()
                    )

                self.enableElement(
                    self.ui.butSwitchDirectionMDM, self.ui.butDeletePreviousMDM
                )

                self.measurementLog.writeLog(
                    [self.data[0][-1], self.data[1][-1], self.data[2][-1]]
                )
                self.updatePlotMDM()
        else:
            self.ui.SRCount.setText("{:.5f}".format(self.singleReadForce))
            if self.homed:
                self.ui.butRecord.setEnabled(True)
            self.singleReadToggle = False

        self.enableElement(self.ui.butSingleRead, self.ui.butTare, self.ui.butConnect)

    def singleReadSkipsUpdate(self) -> None:
        """
        Changes the value of singleReadSkips when textbox is changed
        """
        self.singleReadSkips = self.ui.setLineSkips.value()

    def singleReadLinesForcesUpdate(self) -> None:
        """
        Changes the value of singleReadForces when textbox is changed
        """
        self.singleReadForces = self.ui.setLineReads.value()

    def singleReadStepUpdate(self) -> None:
        """
        Changes the value of stepSizeMDM when textbox is changed
        """
        self.stepSizeMDM = self.ui.setStepSizeMDM.value()
        if self.switchDirectionMDMToggle:
            self.stepSizeMDM = -1 * self.stepSizeMDM

    def readForceMDM(self) -> None:
        self.disableElement(self.ui.butReadForceMDM, self.ui.butSwitchDirectionMDM)
        self.thread_pool.start(self.mainLogWorker.singleRead)

    def switchDirectionMDM(self) -> None:
        self.measurementLog.closeFile()
        del self.measurementLog

        self.readForceMDMToggle = False
        self.enableElement(self.ui.butDeletePreviousMDM)

        if self.switchDirectionMDMToggle:
            self.switchDirectionMDMToggle = False
            del self.txtLogMDM
            self.txtLogMDM = str()
            self.ui.plainTextEdit.clear()
            self.ui.butSwitchDirectionMDM.setText("Switch Direction")

            self.fileMDMOpen = False
            self.ui.butFileMDM.setChecked(False)
            self.ui.butFileMDM.setText("-")
            self.butClear()
            self.enableElement(self.ui.butSwitchManual, self.ui.butConnect)
            self.disableElement(self.ui.butReadForceMDM, self.ui.butSwitchDirectionMDM)

        else:
            self.switchDirectionMDMToggle = True
            self.ui.butSwitchDirectionMDM.setText("Stop")

            self.measurementLog = Logging(
                "".join(self.filePath.split(".")[:-1]) + "_out.csv"
            )
            self.measurementLog.createLogGUI()

            self.stepSizeMDM = -self.stepSizeMDM
            if len(self.data[1]) > 0:
                self.switchDistance: float = self.data[1][-1]
                self.switchForce: float = self.data[2][-1]
            else:
                self.switchDistance, self.switchForce = 0.0, 0.0
            del self.data

            self.data = [[0], [self.switchDistance], [self.switchForce]]

            self.measurementLog.writeLog([self.data[1][-1], self.data[2][-1]])

            self.readForceMDMToggle = True
            del self.txtLogMDM
            self.txtLogMDM = str()
            if re.search(self.reMDMMatch, self.ui.xLabel_2.text()) and re.search(
                self.reMDMMatch, self.ui.yLabel_2.text()
            ):
                xUnit: list[str] = self.ui.xLabel_2.text().split("[")[1].split("]")
                yUnit: list[str] = self.ui.yLabel_2.text().split("[")[1].split("]")
                if len(xUnit) > 0 and len(yUnit) > 0:
                    self.txtLogMDM = (
                        self.txtLogMDM
                        + f"{self.data[1][-1]} {xUnit[0]}, {self.data[2][-1]} {yUnit[0]}"
                    )
            else:
                self.txtLogMDM = (
                    self.txtLogMDM + f"{self.data[1][-1]}, {self.data[2][-1]}"
                )
            self.ui.plainTextEdit.setPlainText(self.txtLogMDM)
            self.plainTextEditScrollbar = self.ui.plainTextEdit.verticalScrollBar()
            self.plainTextEditScrollbar.setValue(self.plainTextEditScrollbar.maximum())

    def butSwitchMDM(self) -> None:
        self.butClear()
        if self.MDMActive:
            self.MDMActive = False
            # visibility
            self.ui.centerGraph.setVisible(True)
            self.ui.MDM.setVisible(False)

            # main ui buttons
            self.enableElement(self.ui.graphOptions, self.ui.butClear)
            if self.sensorConnected and self.homed:
                self.enableElement(self.ui.butRecord)

            # MDM
            self.disableElement(self.ui.MDM)

        else:
            self.MDMActive = True

            # visibility
            self.ui.centerGraph.setVisible(False)
            self.ui.MDM.setVisible(True)

            # main ui buttons
            self.disableElement(
                self.ui.graphOptions,
                self.ui.butSave,
                self.ui.butClear,
                self.ui.butRecord,
            )

            # MDM
            self.ui.MDM.setEnabled(True)

    def plotMDM(self, **kwargs) -> None:
        pg.setConfigOption("foreground", kwargs.pop("clrFg", "k"))
        pg.setConfigOption("background", kwargs.pop("clrBg", "w"))
        # self.ui.graphMDM.setBackground(background=kwargs.pop("clrBg", "w"))
        self.graphMDM1 = self.ui.graphMDM.plot(
            *self.data[1:],
            name=kwargs.pop("nameIn", "Approach"),
            symbol=kwargs.pop("symbolIn", None),
            pen=pg.mkPen(
                {
                    "color": kwargs.pop("colorIn", (0, 0, 255)),
                    "width": kwargs.pop("linewidthIn", 5),
                }
            ),
        )
        self.graphMDM2 = self.ui.graphMDM.plot(
            *self.data[1:],
            name=kwargs.pop("nameOut", "Retraction"),
            symbol=kwargs.pop("symbolOut", None),
            pen=pg.mkPen(
                {
                    "color": kwargs.pop("colorOut", (255, 127, 0)),
                    "width": kwargs.pop("linewidthOut", 5),
                }
            ),
        )
        self.ui.graphMDM.setLabel(
            kwargs.pop("labelyloc", "left"),
            kwargs.pop("labelytxt", self.ui.yLabel_2.text()),
        )
        self.ui.graphMDM.setLabel(
            kwargs.pop("labelxloc", "bottom"),
            kwargs.pop("labelxtxt", self.ui.xLabel_2.text()),
        )

        self.graphMDMLegend = self.ui.graphMDM.addLegend(
            offset=(1, 1), labelTextColor=(255, 255, 255)
        )
        self.graphMDMLegend.addItem(self.graphMDM1, name=self.graphMDM1.name())
        self.graphMDMLegend.addItem(self.graphMDM2, name=self.graphMDM2.name())

        self.ui.graphMDM.setTitle(self.ui.title_2.text(), color=(255, 255, 255))

        self.ui.yLabel_2.textChanged.connect(self.updatePlotMDMYLabel)
        self.ui.xLabel_2.textChanged.connect(self.updatePlotMDMXLabel)

    def updatePlotMDMTitle(self) -> None:
        self.ui.graphMDM.setTitle(self.ui.title_2.text(), color=(255, 255, 255))

    def updatePlotMDMYLabel(self) -> None:
        self.updatePlotLabel(
            graph=self.ui.graphMDM, labelLoc="left", labelTxt=self.ui.yLabel_2.text()
        )

    def updatePlotMDMXLabel(self) -> None:
        self.updatePlotLabel(
            graph=self.ui.graphMDM, labelLoc="bottom", labelTxt=self.ui.xLabel_2.text()
        )

    def updatePlotMDM(self) -> None:
        if self.switchDirectionMDMToggle:
            self.graphMDM2.setData(*self.data[1:])
        else:
            self.graphMDM1.setData(*self.data[1:])

    def butFileMDM(self) -> None:
        """
        Function for what `butFileMDM` has to do.

        - `if fileMDMOpen:` close file
        - `else:` opens dialog box to select/ create a .csv file
        """
        if self.fileMDMOpen:
            if not self.switchDirectionMDMToggle:
                self.switchDirectionMDM()
            else:
                self.disableElement(self.ui.butSwitchDirectionMDM)
            self.switchDirectionMDMToggle = False
            del self.txtLogMDM
            self.txtLogMDM = str()
            self.ui.plainTextEdit.clear()
            self.ui.butSwitchDirectionMDM.setText("Switch Direction")
            self.readForceMDMToggle = False

            self.fileMDMOpen = False
            self.ui.butFileMDM.setChecked(False)
            del self.measurementLog
            self.ui.butFileMDM.setText("-")
            self.butClear()
            self.enableElement(self.ui.butSwitchManual, self.ui.butConnect)
            self.disableElement(self.ui.butReadForceMDM, self.ui.butSwitchDirectionMDM)

        else:
            self.filePath, _ = QtWidgets.QFileDialog.getSaveFileName(
                filter="CSV files (*.csv)"
            )
            if self.filePath != "":
                self.fileMDMOpen = True
                self.ui.butFileMDM.setChecked(True)
                self.measurementLog = Logging(
                    "".join(self.filePath.split(".")[:-1]) + "_in.csv"
                )
                self.measurementLog.createLogGUI()
                self.ui.butFileMDM.setText(
                    *self.filePath.split("/")[-1].split(".")[:-1]
                )
                self.ui.butSwitchManual.setEnabled(False)
                if self.sensorConnected:
                    self.ui.butReadForceMDM.setEnabled(True)
            else:
                self.ui.butFileMDM.setText("-")

    def butDeletePreviousMDM(self) -> None:
        """
        Deletes the previous value in self.data

        main use for when MDM hits other side in capillary bridge experiment, or when the capillary bridge gets broken without being noticed
        """
        # data changes
        for i in range(len(self.data)):
            self.data[i] = self.data[i][:-1]

        # already switched and only 1 value left
        if len(self.data[1]) <= 1 and self.switchDirectionMDMToggle:
            self.disableElement(
                self.ui.butDeletePreviousMDM, self.ui.butSwitchDirectionMDM
            )
        # not switched and no values left
        elif len(self.data[1]) <= 0:
            self.readForceMDMToggle = False
            self.disableElement(
                self.ui.butDeletePreviousMDM, self.ui.butSwitchDirectionMDM
            )
        # there should be a better way to just
        # drop the last value, but this works for now
        # and does not seem to cause much trouble
        self.measurementLog.replaceFile(data=self.data)

        # text box changes
        self.txtLogMDM = str("\n").join(self.txtLogMDM.split("\n")[:-1])
        self.ui.plainTextEdit.setPlainText(self.txtLogMDM)
        self.plainTextEditScrollbar = self.ui.plainTextEdit.verticalScrollBar()
        self.plainTextEditScrollbar.setValue(self.plainTextEditScrollbar.maximum())

        self.updatePlotMDM()

    def setLoadPerCount(self) -> None:
        """
        Changes the value of LoadPerCount when textbox is changed

        Allows for changing the value while still getting live data
        """
        self.sensor.loadPerCount = self.ui.setNewtonPerCount.value()

    def butMove(self) -> None:
        """Handles move button press"""
        self.sensor.cmds.SP(self.ui.setPosition.value())

    def butUpdateVelocity(self) -> None:
        self.velocity = int(self.ui.setVelocity.value())
        self.sensor.cmds.SV(self.velocity)

    def butHome(self) -> None:
        self.ui.errorMessage = [
            "Home Warning",
            "Home Warning",
            "Make sure nothing is obstructing the path downwards.<br>The motor will not stop during homing!",
        ]
        if self.error():
            self.butUpdateVelocity()
            self.sensor.cmds.HM()
            self.homed = True
            self.enableElement(self.ui.butRecord, self.ui.butMove)

    def butForceStop(self) -> None:
        self.homed = False
        self.enableElement(self.ui.butHome)
        self.disableElement(self.ui.butRecord, self.ui.butMove)
        if self.recording:
            self.recording = False
            self.ui.butRecord.setText("Start")
            self.enableElement(
                self.ui.butClear,
                self.ui.butFile,
                self.ui.butTare,
                self.ui.butSave,
                self.ui.butSingleRead,
                self.ui.butSwitchManual,
            )
        try:
            self.sensor.cmds.ST()
        except RuntimeError as e:
            self.ui.errorMessage = [
                e.__class__.__name__,
                str(e),
                "Might have to unplug the adapter and sensor.",
            ]
            self.error()

    def butDisplayTare(self) -> None:
        self.sensor.cmds.TR()

    def butDisplayForce(self) -> None:
        self.sensor.cmds.SF(float(self.ui.setForceApplied.value()))

    def updateUnitDisplay(self) -> None:
        if self.sensorConnected:
            self.sensor.cmds.UU(str(self.ui.setUnitDisplay.text()))

    def swapPositions(self) -> None:
        startPos = self.ui.setStartPos.value()
        endPos = self.ui.setEndPos.value()
        self.ui.setStartPos.setValue(endPos)
        self.ui.setEndPos.setValue(startPos)


class mainLogWorker(QObject, QRunnable):
    startSignal = Signal()
    endSignal = Signal()
    errorSignal = Signal()
    switchXAxisSignal = Signal()
    singleReadStartSignal = Signal()
    singleReadEndSignal = Signal()

    def __init__(self, callerSelf: UserInterface) -> None:
        super().__init__()
        self.callerSelf: UserInterface = callerSelf
        self.logLess: bool = bool()
        self.singleReadForces: int = self.callerSelf.singleReadForces

    def run(self) -> None:
        # mm/s speed of stage
        trueVelocity: float = (self.callerSelf.velocity) / 60

        currentPos: int = self.callerSelf.sensor.cmds.GP()
        startPos: int = self.callerSelf.ui.setStartPos.value()
        endPos: int = self.callerSelf.ui.setEndPos.value()
        Position: float = 0.0

        travelTime: float = abs(endPos - startPos) / trueVelocity
        measurementTime: float = travelTime + self.callerSelf.ui.setTime.value()
        allowTimeSwitch = self.callerSelf.ui.setTime.value() != 0.0
        if currentPos != startPos:
            self.callerSelf.sensor.cmds.SP(startPos)
            # wait until the stage has reached the start position
            sleep(abs(startPos - currentPos) / trueVelocity + 1)
        self.singleReadForces = self.callerSelf.singleReadForces

        _skip: list[float] = [
            self.callerSelf.sensor.cmds.SR()
            for i in range(self.callerSelf.singleReadSkips)
        ]

        self.startSignal.emit()
        self.callerSelf.sensor.cmds.DC(False)
        time = float(0.0)
        self.callerSelf.sensor.T0 = perf_counter_ns()

        # start movement
        self.callerSelf.sensor.cmds.SP(endPos)

        while (time < measurementTime) and self.callerSelf.recording:
            try:
                time = round((perf_counter_ns() - self.callerSelf.sensor.T0) / 1e9, 8)
                if time < travelTime:
                    Position = trueVelocity * time
                elif self.callerSelf.plotIndexX != 0 and allowTimeSwitch:
                    Position = float(abs(endPos - startPos))
                    self.switchXAxisSignal.emit()
                Force = self.read()
                self.callerSelf.data[0].append(time)
                self.callerSelf.data[1].append(Position)
                self.callerSelf.data[2].append(Force)
                if not self.logLess:
                    # logs: t[s], s[mm], F[mN]
                    self.callerSelf.measurementLog.writeLog([time, Position, Force])

                self.singleReadForces = self.callerSelf.singleReadForces

            except ValueError:
                # I know this isn't the best way to deal with it, but it works fine (for now)
                pass

        self.callerSelf.sensor.cmds.DC()
        self.endSignal.emit()

        if self.callerSelf.recording:
            self.callerSelf.threadReachedEnd = True
            self.callerSelf.butRecord()

        if self.logLess:
            # self.callerSelf.unsavedData = self.callerSelf.data
            self.callerSelf.enableElement(self.callerSelf.ui.butSave)

    def read(self) -> float:
        forces: list[float] = [
            self.callerSelf.sensor.ForceFix(self.callerSelf.sensor.cmds.SR())
            for i in range(self.singleReadForces)
        ]
        Force = round(sum(forces) / self.singleReadForces, ndigits=8)
        return Force

    def singleRead(self) -> None:
        self.singleReadStartSignal.emit()
        self.singleReadForces = self.callerSelf.singleReadForces
        _skip: list[float] = [
            self.callerSelf.sensor.ForceFix(self.callerSelf.sensor.cmds.SR())
            for i in range(self.callerSelf.singleReadSkips)
        ]
        self.callerSelf.singleReadForce = self.read()
        self.singleReadEndSignal.emit()


class saveToLog(QObject, QRunnable):
    startSignal = Signal()
    endSignal = Signal()

    def __init__(self, callerSelf: UserInterface) -> None:
        super().__init__()
        self.callerSelf: UserInterface = callerSelf

    def run(self) -> None:
        self.startSignal.emit()
        self.callerSelf.measurementLog.writeLogFull(self.callerSelf.data)
        self.endSignal.emit()


class ForceSensorGUI(ForceSensor, QObject, QRunnable):
    errorSignal = Signal()

    def __init__(
        self, caller: UserInterface, PortName: str | None = None, **kwargs
    ) -> None:
        """
        Class that combines the ForceSensor class with GUI parts and multithreading.

        :param PortName: Portname over which to establish the connection. If None, the this class has to be called again with the port name.
        :type PortName: str | None
        """
        ForceSensor.__init__(self, PortName=PortName, kwargs=kwargs)
        QObject.__init__(self)
        QRunnable.__init__(self)

        self.caller: UserInterface = caller
        self.ui: Ui_MainWindow = caller.ui
        self.failed: bool = False

        if PortName is not None:
            self.tareValue: float = float(self.ui.setGaugeValue.value())
            self.loadPerCount: float = float(self.ui.setNewtonPerCount.value())

            self.PortName: str = PortName

            try:
                self.ser.setPort(self.PortName)
                self.ser.open()
                self.ser.setRTS(False)
                self.ser.setDTR(False)
            except Exception as e:
                self.failed = True
                self.ui.errorMessage = [
                    e.__class__.__name__,
                    str(e),
                    "Check if Port is not already in use.",
                ]
                self.errorSignal.emit()

    def __call__(self, **kwargs) -> None:
        """
        Opens up the serial port, checks the gauge value and makes sure data is available.
        """
        self.tareValue: float = float(self.ui.setGaugeValue.value())
        self.loadPerCount: float = float(self.ui.setNewtonPerCount.value())

        self.PortName: str = self.ui.setPortName.text()

        try:
            self.ser.setPort(self.PortName)
            self.ser.open()
            self.ser.setRTS(False)
            self.ser.setDTR(False)
        except Exception as e:
            self.failed = True
            self.ui.errorMessage = [
                e.__class__.__name__,
                str(e),
                "Check if Port is not already in use.",
            ]
            self.errorSignal.emit()


class ErrorInterface(QtWidgets.QDialog):
    def __init__(self) -> None:
        # roep de __init__() aan van de parent class
        super().__init__()

        self.ui = Ui_errorWindow()
        self.ui.setupUi(self)
        self.ui.ErrorText.setTextFormat(Qt.TextFormat.RichText)

    def __call__(
        self, windowTitle: str, errorText: str, additionalInfo: str | None = None
    ) -> int:
        """
        Enabling the window with the different types.

        :param windowTitle: sets the window title
        :type windowTitle: str
        :param errorText: sets the short error text in the window
        :type errorText: str
        :param additionalInfo: sets the additional information in the window
        :type additionalInfo: str | None

        :return: returns the value of the button pressed, 1 for OK, 0 for Cancel
        :rtype: int
        """
        self.setWindowTitle(windowTitle)
        if additionalInfo is not None:
            self.ui.ErrorText.setText(f"""
<b>{errorText}</b><br>
<br>
{additionalInfo}
""")
        else:
            self.ui.ErrorText.setText(f"<b>{errorText}</b>")

        return self.exec()


def start() -> None:
    """
    Basic main function that starts the GUI

    this function can be recreated to change values set in `UserInterface`

    Function:
    ```
    import sys
    from pyside6 import QtWidgets
    from use_the_force import gui
    def main() -> None:
        app = QtWidgets.QApplication(sys.argv)
        ui = gui.UserInterface()
        ui.show()
        ret = app.exec_()
        sys.exit(ret)
    ```
    """
    app = QtWidgets.QApplication(sys.argv)
    ui = UserInterface()
    ui.show()
    ret = app.exec_()
    sys.exit(ret)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'main_ui.ui'
##
## Created by: Qt User Interface Compiler version 6.8.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractSpinBox, QApplication, QDoubleSpinBox, QFrame,
    QGroupBox, QLabel, QLayout, QLineEdit,
    QMainWindow, QPlainTextEdit, QPushButton, QScrollArea,
    QSizePolicy, QSpinBox, QToolBox, QWidget)

from pyqtgraph import PlotWidget

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(840, 400)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.centralwidget.setMinimumSize(QSize(840, 400))
        self.settings = QScrollArea(self.centralwidget)
        self.settings.setObjectName(u"settings")
        self.settings.setGeometry(QRect(0, 20, 120, 380))
        self.settings.setMinimumSize(QSize(120, 380))
        self.settings.setFrameShape(QFrame.Shape.NoFrame)
        self.settings.setFrameShadow(QFrame.Shadow.Plain)
        self.settings.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.settings.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 120, 380))
        self.toolBox = QToolBox(self.scrollAreaWidgetContents)
        self.toolBox.setObjectName(u"toolBox")
        self.toolBox.setEnabled(True)
        self.toolBox.setGeometry(QRect(1, 0, 118, 379))
        self.toolBox.setFrameShape(QFrame.Shape.NoFrame)
        self.sensorOptions = QWidget()
        self.sensorOptions.setObjectName(u"sensorOptions")
        self.sensorOptions.setEnabled(True)
        self.sensorOptions.setGeometry(QRect(0, 0, 118, 259))
        self.labPortName = QLabel(self.sensorOptions)
        self.labPortName.setObjectName(u"labPortName")
        self.labPortName.setGeometry(QRect(1, -1, 113, 20))
        self.setPortName = QLineEdit(self.sensorOptions)
        self.setPortName.setObjectName(u"setPortName")
        self.setPortName.setEnabled(True)
        self.setPortName.setGeometry(QRect(1, 19, 113, 20))
        self.labGaugeValue = QLabel(self.sensorOptions)
        self.labGaugeValue.setObjectName(u"labGaugeValue")
        self.labGaugeValue.setGeometry(QRect(1, 39, 113, 20))
        self.labNewtonPerCount = QLabel(self.sensorOptions)
        self.labNewtonPerCount.setObjectName(u"labNewtonPerCount")
        self.labNewtonPerCount.setGeometry(QRect(1, 79, 113, 20))
        self.butConnect = QPushButton(self.sensorOptions)
        self.butConnect.setObjectName(u"butConnect")
        self.butConnect.setGeometry(QRect(1, 119, 113, 22))
        self.butConnect.setCheckable(True)
        self.butConnect.setChecked(False)
        self.butConnect.setFlat(False)
        self.setNewtonPerCount = QDoubleSpinBox(self.sensorOptions)
        self.setNewtonPerCount.setObjectName(u"setNewtonPerCount")
        self.setNewtonPerCount.setEnabled(False)
        self.setNewtonPerCount.setGeometry(QRect(1, 99, 113, 20))
        self.setNewtonPerCount.setDecimals(12)
        self.setNewtonPerCount.setMinimum(-9999999999999999635896294965248.000000000000000)
        self.setNewtonPerCount.setMaximum(9999999999999999455752309870428160.000000000000000)
        self.setNewtonPerCount.setSingleStep(0.000001000000000)
        self.setNewtonPerCount.setStepType(QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.setNewtonPerCount.setValue(1.000000000000000)
        self.displayOptions = QGroupBox(self.sensorOptions)
        self.displayOptions.setObjectName(u"displayOptions")
        self.displayOptions.setGeometry(QRect(1, 139, 113, 119))
        font = QFont()
        font.setBold(True)
        self.displayOptions.setFont(font)
        self.setUnitDisplay = QLineEdit(self.displayOptions)
        self.setUnitDisplay.setObjectName(u"setUnitDisplay")
        self.setUnitDisplay.setEnabled(False)
        self.setUnitDisplay.setGeometry(QRect(1, 100, 109, 20))
        font1 = QFont()
        font1.setBold(False)
        self.setUnitDisplay.setFont(font1)
        self.setUnitDisplay.setMaxLength(8)
        self.setForceApplied = QDoubleSpinBox(self.displayOptions)
        self.setForceApplied.setObjectName(u"setForceApplied")
        self.setForceApplied.setEnabled(False)
        self.setForceApplied.setGeometry(QRect(1, 60, 109, 20))
        self.setForceApplied.setFont(font1)
        self.setForceApplied.setDecimals(8)
        self.setForceApplied.setMinimum(0.000000000000000)
        self.setForceApplied.setMaximum(999999.000000000000000)
        self.setForceApplied.setStepType(QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.setForceApplied.setValue(1.000000000000000)
        self.butTareDisplay = QPushButton(self.displayOptions)
        self.butTareDisplay.setObjectName(u"butTareDisplay")
        self.butTareDisplay.setEnabled(False)
        self.butTareDisplay.setGeometry(QRect(1, 19, 109, 22))
        self.butTareDisplay.setFont(font1)
        self.labForceApplied = QLabel(self.displayOptions)
        self.labForceApplied.setObjectName(u"labForceApplied")
        self.labForceApplied.setGeometry(QRect(1, 41, 109, 19))
        self.labForceApplied.setFont(font1)
        self.labUnitDisplay = QLabel(self.displayOptions)
        self.labUnitDisplay.setObjectName(u"labUnitDisplay")
        self.labUnitDisplay.setGeometry(QRect(1, 80, 109, 20))
        self.labUnitDisplay.setFont(font1)
        self.setGaugeValue = QSpinBox(self.sensorOptions)
        self.setGaugeValue.setObjectName(u"setGaugeValue")
        self.setGaugeValue.setEnabled(False)
        self.setGaugeValue.setGeometry(QRect(1, 59, 113, 20))
        self.setGaugeValue.setMinimum(-999999999)
        self.setGaugeValue.setMaximum(999999999)
        self.toolBox.addItem(self.sensorOptions, u"Sensor")
        self.moveOptions = QWidget()
        self.moveOptions.setObjectName(u"moveOptions")
        self.moveOptions.setGeometry(QRect(0, 0, 98, 28))
        self.labVelocity = QLabel(self.moveOptions)
        self.labVelocity.setObjectName(u"labVelocity")
        self.labVelocity.setGeometry(QRect(1, -1, 113, 20))
        self.setPosition = QSpinBox(self.moveOptions)
        self.setPosition.setObjectName(u"setPosition")
        self.setPosition.setGeometry(QRect(1, 81, 113, 20))
        self.setPosition.setMinimum(1)
        self.setPosition.setMaximum(46)
        self.butMove = QPushButton(self.moveOptions)
        self.butMove.setObjectName(u"butMove")
        self.butMove.setEnabled(False)
        self.butMove.setGeometry(QRect(1, 101, 113, 22))
        self.labPosition = QLabel(self.moveOptions)
        self.labPosition.setObjectName(u"labPosition")
        self.labPosition.setGeometry(QRect(1, 61, 113, 20))
        self.setVelocity = QSpinBox(self.moveOptions)
        self.setVelocity.setObjectName(u"setVelocity")
        self.setVelocity.setGeometry(QRect(1, 19, 113, 20))
        self.setVelocity.setMinimum(1)
        self.setVelocity.setMaximum(120)
        self.setVelocity.setValue(60)
        self.butUpdateVelocity = QPushButton(self.moveOptions)
        self.butUpdateVelocity.setObjectName(u"butUpdateVelocity")
        self.butUpdateVelocity.setEnabled(False)
        self.butUpdateVelocity.setGeometry(QRect(1, 39, 113, 22))
        self.GroupMovementPattern = QGroupBox(self.moveOptions)
        self.GroupMovementPattern.setObjectName(u"GroupMovementPattern")
        self.GroupMovementPattern.setGeometry(QRect(1, 123, 113, 120))
        self.GroupMovementPattern.setFont(font)
        self.GroupMovementPattern.setFlat(True)
        self.labStartPos = QLabel(self.GroupMovementPattern)
        self.labStartPos.setObjectName(u"labStartPos")
        self.labStartPos.setGeometry(QRect(1, 13, 111, 20))
        self.labStartPos.setFont(font1)
        self.setStartPos = QSpinBox(self.GroupMovementPattern)
        self.setStartPos.setObjectName(u"setStartPos")
        self.setStartPos.setGeometry(QRect(1, 33, 111, 20))
        self.setStartPos.setFont(font1)
        self.setStartPos.setMinimum(1)
        self.setStartPos.setMaximum(46)
        self.setStartPos.setValue(46)
        self.labEndPos = QLabel(self.GroupMovementPattern)
        self.labEndPos.setObjectName(u"labEndPos")
        self.labEndPos.setGeometry(QRect(1, 53, 111, 20))
        self.labEndPos.setFont(font1)
        self.setEndPos = QSpinBox(self.GroupMovementPattern)
        self.setEndPos.setObjectName(u"setEndPos")
        self.setEndPos.setGeometry(QRect(1, 73, 111, 20))
        self.setEndPos.setFont(font1)
        self.setEndPos.setMinimum(1)
        self.setEndPos.setMaximum(46)
        self.setEndPos.setValue(46)
        self.butSwapPositions = QPushButton(self.GroupMovementPattern)
        self.butSwapPositions.setObjectName(u"butSwapPositions")
        self.butSwapPositions.setGeometry(QRect(0, 94, 113, 22))
        self.butSwapPositions.setFont(font1)
        self.butSwapPositions.setLocale(QLocale(QLocale.English, QLocale.UnitedKingdom))
        self.toolBox.addItem(self.moveOptions, u"Movement")
        self.measurementOptions = QWidget()
        self.measurementOptions.setObjectName(u"measurementOptions")
        self.measurementOptions.setEnabled(True)
        self.measurementOptions.setGeometry(QRect(0, 0, 98, 28))
        self.fileLabel = QLabel(self.measurementOptions)
        self.fileLabel.setObjectName(u"fileLabel")
        self.fileLabel.setGeometry(QRect(1, -1, 113, 20))
        self.butFile = QPushButton(self.measurementOptions)
        self.butFile.setObjectName(u"butFile")
        self.butFile.setGeometry(QRect(1, 19, 113, 22))
        self.butFile.setCheckable(True)
        self.labLineSkips = QLabel(self.measurementOptions)
        self.labLineSkips.setObjectName(u"labLineSkips")
        self.labLineSkips.setGeometry(QRect(1, 41, 113, 20))
        self.labLineSkips.setFont(font1)
        self.setLineSkips = QSpinBox(self.measurementOptions)
        self.setLineSkips.setObjectName(u"setLineSkips")
        self.setLineSkips.setGeometry(QRect(1, 61, 113, 20))
        self.setLineSkips.setFont(font1)
        self.setLineSkips.setMinimum(0)
        self.setLineSkips.setMaximum(999999999)
        self.setLineSkips.setValue(0)
        self.setLineReads = QSpinBox(self.measurementOptions)
        self.setLineReads.setObjectName(u"setLineReads")
        self.setLineReads.setGeometry(QRect(1, 101, 113, 20))
        self.setLineReads.setFont(font1)
        self.setLineReads.setMinimum(1)
        self.setLineReads.setMaximum(999999999)
        self.setLineReads.setValue(1)
        self.labLineReads = QLabel(self.measurementOptions)
        self.labLineReads.setObjectName(u"labLineReads")
        self.labLineReads.setGeometry(QRect(1, 81, 113, 20))
        self.labLineReads.setFont(font1)
        self.timeLabel = QLabel(self.measurementOptions)
        self.timeLabel.setObjectName(u"timeLabel")
        self.timeLabel.setGeometry(QRect(1, 121, 113, 20))
        self.setTime = QDoubleSpinBox(self.measurementOptions)
        self.setTime.setObjectName(u"setTime")
        self.setTime.setGeometry(QRect(1, 141, 113, 20))
        self.setTime.setDecimals(6)
        self.setTime.setMinimum(0.000000000000000)
        self.setTime.setMaximum(9999999.000000000000000)
        self.setTime.setValue(0.000000000000000)
        self.toolBox.addItem(self.measurementOptions, u"Measurement")
        self.graphOptions = QWidget()
        self.graphOptions.setObjectName(u"graphOptions")
        self.graphOptions.setEnabled(True)
        self.graphOptions.setGeometry(QRect(0, 0, 98, 28))
        self.yLabel = QLineEdit(self.graphOptions)
        self.yLabel.setObjectName(u"yLabel")
        self.yLabel.setEnabled(True)
        self.yLabel.setGeometry(QRect(1, 59, 113, 20))
        self.yLabelLabel = QLabel(self.graphOptions)
        self.yLabelLabel.setObjectName(u"yLabelLabel")
        self.yLabelLabel.setGeometry(QRect(1, 39, 113, 20))
        self.xLimLabel = QLabel(self.graphOptions)
        self.xLimLabel.setObjectName(u"xLimLabel")
        self.xLimLabel.setGeometry(QRect(1, 79, 113, 20))
        self.title = QLineEdit(self.graphOptions)
        self.title.setObjectName(u"title")
        self.title.setGeometry(QRect(1, 19, 113, 20))
        self.titleLabel = QLabel(self.graphOptions)
        self.titleLabel.setObjectName(u"titleLabel")
        self.titleLabel.setGeometry(QRect(1, -1, 113, 20))
        self.plotTimerIntervalLabel = QLabel(self.graphOptions)
        self.plotTimerIntervalLabel.setObjectName(u"plotTimerIntervalLabel")
        self.plotTimerIntervalLabel.setGeometry(QRect(1, 219, 113, 20))
        self.setPlotTimerInterval = QSpinBox(self.graphOptions)
        self.setPlotTimerInterval.setObjectName(u"setPlotTimerInterval")
        self.setPlotTimerInterval.setGeometry(QRect(1, 239, 113, 20))
        self.setPlotTimerInterval.setMinimum(1)
        self.setPlotTimerInterval.setMaximum(100000)
        self.setPlotTimerInterval.setSingleStep(100)
        self.setPlotTimerInterval.setValue(200)
        self.xLimSet = QSpinBox(self.graphOptions)
        self.xLimSet.setObjectName(u"xLimSet")
        self.xLimSet.setGeometry(QRect(1, 99, 113, 20))
        self.xLimSet.setMinimum(-1)
        self.xLimSet.setMaximum(0)
        self.toolBox.addItem(self.graphOptions, u"Graph")
        self.settings.setWidget(self.scrollAreaWidgetContents)
        self.centerGraph = QWidget(self.centralwidget)
        self.centerGraph.setObjectName(u"centerGraph")
        self.centerGraph.setEnabled(True)
        self.centerGraph.setGeometry(QRect(120, 0, 600, 400))
        self.centerGraph.setMinimumSize(QSize(600, 400))
        self.graph1 = PlotWidget(self.centerGraph)
        self.graph1.setObjectName(u"graph1")
        self.graph1.setEnabled(True)
        self.graph1.setGeometry(QRect(0, 0, 600, 400))
        self.graph1.setFrameShape(QFrame.Shape.StyledPanel)
        self.graph1.setFrameShadow(QFrame.Shadow.Sunken)
        self.rightWidget = QWidget(self.centralwidget)
        self.rightWidget.setObjectName(u"rightWidget")
        self.rightWidget.setGeometry(QRect(720, 20, 120, 380))
        self.rightWidget.setMinimumSize(QSize(120, 380))
        self.butRecord = QPushButton(self.rightWidget)
        self.butRecord.setObjectName(u"butRecord")
        self.butRecord.setEnabled(False)
        self.butRecord.setGeometry(QRect(1, -1, 118, 24))
        self.butRecord.setCheckable(True)
        self.butSave = QPushButton(self.rightWidget)
        self.butSave.setObjectName(u"butSave")
        self.butSave.setEnabled(False)
        self.butSave.setGeometry(QRect(1, 47, 118, 24))
        self.butClear = QPushButton(self.rightWidget)
        self.butClear.setObjectName(u"butClear")
        self.butClear.setGeometry(QRect(1, 23, 118, 24))
        self.butSingleRead = QPushButton(self.rightWidget)
        self.butSingleRead.setObjectName(u"butSingleRead")
        self.butSingleRead.setEnabled(False)
        self.butSingleRead.setGeometry(QRect(1, 255, 118, 24))
        self.butSwitchManual = QPushButton(self.rightWidget)
        self.butSwitchManual.setObjectName(u"butSwitchManual")
        self.butSwitchManual.setEnabled(True)
        self.butSwitchManual.setGeometry(QRect(0, 339, 118, 40))
        self.butSwitchManual.setFont(font1)
        self.butForceStop = QPushButton(self.rightWidget)
        self.butForceStop.setObjectName(u"butForceStop")
        self.butForceStop.setEnabled(False)
        self.butForceStop.setGeometry(QRect(1, 119, 118, 24))
        self.butTare = QPushButton(self.rightWidget)
        self.butTare.setObjectName(u"butTare")
        self.butTare.setEnabled(False)
        self.butTare.setGeometry(QRect(1, 71, 118, 24))
        self.butTare.setCheckable(True)
        self.butHome = QPushButton(self.rightWidget)
        self.butHome.setObjectName(u"butHome")
        self.butHome.setEnabled(False)
        self.butHome.setGeometry(QRect(1, 95, 118, 24))
        self.SRCount = QLineEdit(self.rightWidget)
        self.SRCount.setObjectName(u"SRCount")
        self.SRCount.setGeometry(QRect(1, 279, 118, 20))
        self.SRCount.setReadOnly(True)
        self.settingsLabel = QLabel(self.centralwidget)
        self.settingsLabel.setObjectName(u"settingsLabel")
        self.settingsLabel.setGeometry(QRect(0, 0, 120, 20))
        self.settingsLabel.setMinimumSize(QSize(120, 20))
        self.buttonsLabel = QLabel(self.centralwidget)
        self.buttonsLabel.setObjectName(u"buttonsLabel")
        self.buttonsLabel.setGeometry(QRect(721, 0, 120, 20))
        self.buttonsLabel.setMinimumSize(QSize(120, 20))
        self.MDM = QWidget(self.centralwidget)
        self.MDM.setObjectName(u"MDM")
        self.MDM.setEnabled(True)
        self.MDM.setGeometry(QRect(121, 0, 598, 400))
        self.MDM.setMinimumSize(QSize(598, 400))
        self.MDM.setAutoFillBackground(True)
        self.labMDM = QLabel(self.MDM)
        self.labMDM.setObjectName(u"labMDM")
        self.labMDM.setGeometry(QRect(0, 0, 600, 20))
        self.labMDM.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.butReadForceMDM = QPushButton(self.MDM)
        self.butReadForceMDM.setObjectName(u"butReadForceMDM")
        self.butReadForceMDM.setEnabled(False)
        self.butReadForceMDM.setGeometry(QRect(120, 319, 238, 80))
        self.extraSettingsBoxMDM = QGroupBox(self.MDM)
        self.extraSettingsBoxMDM.setObjectName(u"extraSettingsBoxMDM")
        self.extraSettingsBoxMDM.setGeometry(QRect(1, 20, 118, 298))
        self.extraSettingsBoxMDM.setFont(font)
        self.labelFilenameMDM = QLabel(self.extraSettingsBoxMDM)
        self.labelFilenameMDM.setObjectName(u"labelFilenameMDM")
        self.labelFilenameMDM.setGeometry(QRect(1, 13, 113, 20))
        self.labelFilenameMDM.setFont(font1)
        self.labStepSizeMDM = QLabel(self.extraSettingsBoxMDM)
        self.labStepSizeMDM.setObjectName(u"labStepSizeMDM")
        self.labStepSizeMDM.setGeometry(QRect(1, 53, 113, 20))
        self.labStepSizeMDM.setFont(font1)
        self.butFileMDM = QPushButton(self.extraSettingsBoxMDM)
        self.butFileMDM.setObjectName(u"butFileMDM")
        self.butFileMDM.setGeometry(QRect(1, 33, 113, 22))
        self.butFileMDM.setFont(font1)
        self.butFileMDM.setCheckable(True)
        self.setStepSizeMDM = QDoubleSpinBox(self.extraSettingsBoxMDM)
        self.setStepSizeMDM.setObjectName(u"setStepSizeMDM")
        self.setStepSizeMDM.setGeometry(QRect(1, 73, 113, 20))
        self.setStepSizeMDM.setFont(font1)
        self.setStepSizeMDM.setDecimals(12)
        self.setStepSizeMDM.setMinimum(0.000000000000000)
        self.setStepSizeMDM.setMaximum(99.989999999999995)
        self.setStepSizeMDM.setSingleStep(0.000001000000000)
        self.setStepSizeMDM.setStepType(QAbstractSpinBox.StepType.AdaptiveDecimalStepType)
        self.setStepSizeMDM.setValue(0.050000000000000)
        self.yLabelLabel_2 = QLabel(self.extraSettingsBoxMDM)
        self.yLabelLabel_2.setObjectName(u"yLabelLabel_2")
        self.yLabelLabel_2.setGeometry(QRect(1, 213, 113, 20))
        self.yLabelLabel_2.setFont(font1)
        self.xLabel_2 = QLineEdit(self.extraSettingsBoxMDM)
        self.xLabel_2.setObjectName(u"xLabel_2")
        self.xLabel_2.setEnabled(True)
        self.xLabel_2.setGeometry(QRect(1, 273, 113, 20))
        self.xLabel_2.setFont(font1)
        self.yLabel_2 = QLineEdit(self.extraSettingsBoxMDM)
        self.yLabel_2.setObjectName(u"yLabel_2")
        self.yLabel_2.setEnabled(True)
        self.yLabel_2.setGeometry(QRect(1, 233, 113, 20))
        self.yLabel_2.setFont(font1)
        self.xLabelLabel_2 = QLabel(self.extraSettingsBoxMDM)
        self.xLabelLabel_2.setObjectName(u"xLabelLabel_2")
        self.xLabelLabel_2.setGeometry(QRect(1, 253, 113, 20))
        self.xLabelLabel_2.setFont(font1)
        self.title_2 = QLineEdit(self.extraSettingsBoxMDM)
        self.title_2.setObjectName(u"title_2")
        self.title_2.setEnabled(True)
        self.title_2.setGeometry(QRect(1, 193, 113, 20))
        self.title_2.setFont(font1)
        self.titleLabel_2 = QLabel(self.extraSettingsBoxMDM)
        self.titleLabel_2.setObjectName(u"titleLabel_2")
        self.titleLabel_2.setGeometry(QRect(1, 173, 113, 20))
        self.titleLabel_2.setFont(font1)
        self.graphMDM = PlotWidget(self.MDM)
        self.graphMDM.setObjectName(u"graphMDM")
        self.graphMDM.setEnabled(True)
        self.graphMDM.setGeometry(QRect(120, 20, 477, 298))
        self.graphMDM.setFrameShape(QFrame.Shape.StyledPanel)
        self.graphMDM.setFrameShadow(QFrame.Shadow.Plain)
        self.butSwitchDirectionMDM = QPushButton(self.MDM)
        self.butSwitchDirectionMDM.setObjectName(u"butSwitchDirectionMDM")
        self.butSwitchDirectionMDM.setEnabled(False)
        self.butSwitchDirectionMDM.setGeometry(QRect(1, 319, 118, 80))
        self.plainTextEdit = QPlainTextEdit(self.MDM)
        self.plainTextEdit.setObjectName(u"plainTextEdit")
        self.plainTextEdit.setGeometry(QRect(359, 319, 238, 56))
        self.plainTextEdit.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.plainTextEdit.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.plainTextEdit.setReadOnly(True)
        self.butDeletePreviousMDM = QPushButton(self.MDM)
        self.butDeletePreviousMDM.setObjectName(u"butDeletePreviousMDM")
        self.butDeletePreviousMDM.setEnabled(False)
        self.butDeletePreviousMDM.setGeometry(QRect(359, 375, 238, 24))
        self.graphMDM.raise_()
        self.labMDM.raise_()
        self.butReadForceMDM.raise_()
        self.extraSettingsBoxMDM.raise_()
        self.butSwitchDirectionMDM.raise_()
        self.plainTextEdit.raise_()
        self.butDeletePreviousMDM.raise_()
        MainWindow.setCentralWidget(self.centralwidget)
        self.settings.raise_()
        self.rightWidget.raise_()
        self.settingsLabel.raise_()
        self.buttonsLabel.raise_()
        self.centerGraph.raise_()
        self.MDM.raise_()

        self.retranslateUi(MainWindow)

        self.toolBox.setCurrentIndex(0)
        self.butConnect.setDefault(False)


        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"Use the Force!", None))
        self.labPortName.setText(QCoreApplication.translate("MainWindow", u"Port Name", None))
        self.setPortName.setText(QCoreApplication.translate("MainWindow", u"No Port Found", None))
        self.labGaugeValue.setText(QCoreApplication.translate("MainWindow", u"Gauge Value", None))
        self.labNewtonPerCount.setText(QCoreApplication.translate("MainWindow", u"Load per Count", None))
        self.butConnect.setText(QCoreApplication.translate("MainWindow", u"Connect", None))
        self.displayOptions.setTitle(QCoreApplication.translate("MainWindow", u"Display Options", None))
        self.setUnitDisplay.setPlaceholderText(QCoreApplication.translate("MainWindow", u" mN", None))
        self.butTareDisplay.setText(QCoreApplication.translate("MainWindow", u"Tare Display Value", None))
        self.labForceApplied.setText(QCoreApplication.translate("MainWindow", u"True Load Value:", None))
        self.labUnitDisplay.setText(QCoreApplication.translate("MainWindow", u"Display Unit:", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.sensorOptions), QCoreApplication.translate("MainWindow", u"Sensor", None))
        self.labVelocity.setText(QCoreApplication.translate("MainWindow", u"Velocity", None))
        self.butMove.setText(QCoreApplication.translate("MainWindow", u"Move", None))
        self.labPosition.setText(QCoreApplication.translate("MainWindow", u"Position", None))
        self.butUpdateVelocity.setText(QCoreApplication.translate("MainWindow", u"Update Velocity", None))
        self.GroupMovementPattern.setTitle(QCoreApplication.translate("MainWindow", u"Movement Pattern", None))
        self.labStartPos.setText(QCoreApplication.translate("MainWindow", u"Start Position", None))
        self.labEndPos.setText(QCoreApplication.translate("MainWindow", u"End Position", None))
        self.butSwapPositions.setText(QCoreApplication.translate("MainWindow", u"Swap Positions", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.moveOptions), QCoreApplication.translate("MainWindow", u"Movement", None))
        self.fileLabel.setText(QCoreApplication.translate("MainWindow", u"File", None))
        self.butFile.setText(QCoreApplication.translate("MainWindow", u"-", None))
        self.labLineSkips.setText(QCoreApplication.translate("MainWindow", u"Line Skips", None))
        self.labLineReads.setText(QCoreApplication.translate("MainWindow", u"Line Reads (average)", None))
        self.timeLabel.setText(QCoreApplication.translate("MainWindow", u"Extra Duration (s)", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.measurementOptions), QCoreApplication.translate("MainWindow", u"Measurement", None))
        self.yLabel.setText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.yLabel.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.yLabelLabel.setText(QCoreApplication.translate("MainWindow", u"y-Label", None))
        self.xLimLabel.setText(QCoreApplication.translate("MainWindow", u"x-Scope", None))
        self.title.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Force-Displacement Measurement", None))
        self.titleLabel.setText(QCoreApplication.translate("MainWindow", u"Title", None))
        self.plotTimerIntervalLabel.setText(QCoreApplication.translate("MainWindow", u"Refresh Time (ms)", None))
        self.toolBox.setItemText(self.toolBox.indexOf(self.graphOptions), QCoreApplication.translate("MainWindow", u"Graph", None))
        self.butRecord.setText(QCoreApplication.translate("MainWindow", u"Start", None))
        self.butSave.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.butClear.setText(QCoreApplication.translate("MainWindow", u"Clear", None))
        self.butSingleRead.setText(QCoreApplication.translate("MainWindow", u"Single Read", None))
        self.butSwitchManual.setText(QCoreApplication.translate("MainWindow", u"Switch to Manual\n"
"Displacement Mode", None))
        self.butForceStop.setText(QCoreApplication.translate("MainWindow", u"Force Stop", None))
        self.butTare.setText(QCoreApplication.translate("MainWindow", u"Tare", None))
        self.butHome.setText(QCoreApplication.translate("MainWindow", u"Home", None))
        self.SRCount.setText(QCoreApplication.translate("MainWindow", u"0", None))
        self.settingsLabel.setText(QCoreApplication.translate("MainWindow", u"<html><head/><body><p align=\"center\"><span style=\" font-weight:700;\">Settings</span></p></body></html>", None))
        self.buttonsLabel.setText(QCoreApplication.translate("MainWindow", u"<html><head/><body><p align=\"center\"><span style=\" font-weight:700;\">Buttons</span></p></body></html>", None))
        self.labMDM.setText(QCoreApplication.translate("MainWindow", u"<html><head/><body><p align=\"center\"><span style=\" font-weight:700;\">Manual Displacement Mode</span></p></body></html>", None))
        self.butReadForceMDM.setText(QCoreApplication.translate("MainWindow", u"Read Force", None))
        self.extraSettingsBoxMDM.setTitle(QCoreApplication.translate("MainWindow", u"MDM Settings", None))
        self.labelFilenameMDM.setText(QCoreApplication.translate("MainWindow", u"Filename", None))
        self.labStepSizeMDM.setText(QCoreApplication.translate("MainWindow", u"Step Size (mm)", None))
        self.butFileMDM.setText(QCoreApplication.translate("MainWindow", u"-", None))
        self.yLabelLabel_2.setText(QCoreApplication.translate("MainWindow", u"y-Label", None))
        self.xLabel_2.setText(QCoreApplication.translate("MainWindow", u"Displacement [mm]", None))
        self.xLabel_2.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Displacement [mm]", None))
        self.yLabel_2.setText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.yLabel_2.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Force [mN]", None))
        self.xLabelLabel_2.setText(QCoreApplication.translate("MainWindow", u"x-Label", None))
        self.title_2.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Manual Force-Displacement Curve", None))
        self.titleLabel_2.setText(QCoreApplication.translate("MainWindow", u"Title", None))
        self.butSwitchDirectionMDM.setText(QCoreApplication.translate("MainWindow", u"Switch Direction", None))
        self.butDeletePreviousMDM.setText(QCoreApplication.translate("MainWindow", u"Delete Previous Reading", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'error_ui.ui'
##
## Created by: Qt User Interface Compiler version 6.8.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QApplication, QDialog, QDialogButtonBox,
    QLabel, QLayout, QSizePolicy, QVBoxLayout,
    QWidget)

__all__ = [
    "Ui_errorWindow"
]

class Ui_errorWindow(object):
    def setupUi(self, errorWindow):
        if not errorWindow.objectName():
            errorWindow.setObjectName(u"errorWindow")
        errorWindow.setWindowModality(Qt.WindowModality.WindowModal)
        
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(10)
        sizePolicy.setHeightForWidth(errorWindow.sizePolicy().hasHeightForWidth())
        errorWindow.setSizePolicy(sizePolicy)
        errorWindow.setMinimumSize(QSize(400, 100))
        errorWindow.setSizeIncrement(QSize(0, 20))
        errorWindow.setBaseSize(QSize(400, 100))
        errorWindow.setWindowOpacity(1.000000000000000)
        self.verticalLayoutWidget = QWidget(errorWindow)
        self.verticalLayoutWidget.setObjectName(u"verticalLayoutWidget")
        self.verticalLayoutWidget.setGeometry(QRect(-1, 9, 401, 92))
        self.verticalLayout = QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setSizeConstraint(QLayout.SizeConstraint.SetMinimumSize)
        self.verticalLayout.setContentsMargins(10, 0, 10, 0)
        self.ErrorText = QLabel(self.verticalLayoutWidget)
        self.ErrorText.setObjectName(u"ErrorText")
        self.ErrorText.setEnabled(True)
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.ErrorText.sizePolicy().hasHeightForWidth())
        self.ErrorText.setSizePolicy(sizePolicy1)
        self.ErrorText.setSizeIncrement(QSize(0, 20))
        self.ErrorText.setTextFormat(Qt.TextFormat.RichText)
        self.ErrorText.setScaledContents(True)
        self.ErrorText.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.ErrorText.setWordWrap(True)

        self.verticalLayout.addWidget(self.ErrorText)

        self.ErrorButtons = QDialogButtonBox(self.verticalLayoutWidget)
        self.ErrorButtons.setObjectName(u"ErrorButtons")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.ErrorButtons.sizePolicy().hasHeightForWidth())
        self.ErrorButtons.setSizePolicy(sizePolicy2)
        self.ErrorButtons.setSizeIncrement(QSize(0, 0))
        self.ErrorButtons.setOrientation(Qt.Orientation.Horizontal)
        self.ErrorButtons.setStandardButtons(QDialogButtonBox.StandardButton.Cancel|QDialogButtonBox.StandardButton.Ok)
        self.ErrorButtons.setCenterButtons(False)

        self.verticalLayout.addWidget(self.ErrorButtons)

        self.verticalLayout.setStretch(1, 2000)

        self.retranslateUi(errorWindow)
        self.ErrorButtons.accepted.connect(errorWindow.accept)
        self.ErrorButtons.rejected.connect(errorWindow.reject)

        QMetaObject.connectSlotsByName(errorWindow)
    # setupUi

    def retranslateUi(self, errorWindow):
        errorWindow.setWindowTitle(QCoreApplication.translate("errorWindow", u"An Error Occured", None))
        self.ErrorText.setText(QCoreApplication.translate("errorWindow", u"<html><head/><body><p><span style=\" font-weight:700;\">An Error Occured...</span></p><p>An error occured so badly that the error message never loaded.</p></body></html>", None))
    # retranslateUi

